"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const stream_1 = __importDefault(require("stream"));
const util_1 = __importDefault(require("util"));
const command_1 = __importDefault(require("@moneyforward/command"));
const stream_util_1 = require("@moneyforward/stream-util");
const sca_action_core_1 = __importStar(require("@moneyforward/sca-action-core"));
const debug = util_1.default.debuglog('@moneyforward/code-review-action-rails_best_practices-plugin');
let Analyzer = /** @class */ (() => {
    class Analyzer extends sca_action_core_1.default {
        constructor(...args) {
            super(Analyzer.command, args.map(String).concat(['-f', 'json', '--silent']), undefined, 2, undefined, 'Rails Best Practices');
        }
        static buildOutputFileOptions() {
            const directory = fs_1.default.mkdtempSync(path_1.default.join(os_1.default.tmpdir(), `${Analyzer.command}-`));
            const file = path_1.default.join(directory, 'output.json');
            return ['--output-file', file];
        }
        async prepare() {
            console.log(`::group::Installing gems...`);
            try {
                await new sca_action_core_1.installer.RubyGemsInstaller(true).execute([Analyzer.command]);
            }
            finally {
                console.log(`::endgroup::`);
            }
        }
        createTransformStreams() {
            return [
                new stream_util_1.transform.JSON(),
                new stream_1.default.Transform({
                    objectMode: true,
                    transform: function (result, encoding, done) {
                        debug(`Detected %d problem(s).`, result.length);
                        for (const error of result)
                            this.push({
                                file: error.filename,
                                line: error.line_number,
                                column: undefined,
                                severity: 'error',
                                message: error.message,
                                code: undefined
                            });
                        this.push(null);
                        done();
                    }
                })
            ];
        }
        get Command() {
            return class extends command_1.default {
                constructor(command, args = [], options = {}, iterate, exitStatusThreshold = 1, argumentsSizeMargin = 0) {
                    super(command, args, options, iterate, exitStatusThreshold, argumentsSizeMargin + Analyzer.buildOutputFileOptions().map(command_1.default.sizeOf).reduce((previous, current) => previous + current));
                }
                async configureArguments(args) {
                    const [optionName, file] = Analyzer.buildOutputFileOptions();
                    await fs_1.default.promises.writeFile(file, '');
                    return this.args.concat(optionName, file, ...args);
                }
            };
        }
        pipeline(stdout, writable, ...[, args]) {
            const [optionName, file] = args.slice(this.args.length);
            debug('%s %s', optionName, file);
            const transformers = this.createTransformStreams() || [];
            return new Promise((resolve, reject) => {
                const watcher = fs_1.default.watch(file).on('error', reject).once('change', (eventType, filename) => {
                    const stat = fs_1.default.statSync(file);
                    debug('%s %s (%d bytes)', eventType, filename, stat.size);
                    watcher.close();
                    resolve([fs_1.default.createReadStream(file), ...transformers.concat(writable)]);
                });
            });
        }
    }
    Analyzer.command = 'rails_best_practices';
    return Analyzer;
})();
exports.default = Analyzer;
//# sourceMappingURL=index.js.map